
#include "WidgetMarginLayoutChildren.h"
#include "Widget.h"
#include <Windows.h>

namespace ghost{

	namespace widget{

		namespace{
			bool CheckMarginLayoutAttributes_(int attributes)
			{
				return 0 != ((MARGIN_LAYOUT_KEEP_LEFT|MARGIN_LAYOUT_KEEP_TOP|MARGIN_LAYOUT_KEEP_RIGHT|MARGIN_LAYOUT_KEEP_BOTTOM) & attributes);
			}
		} // namespace

		MarginLayoutChildren::MarginLayoutChildren()
		{
			::SetRectEmpty(&oldRect_);
		}

		bool MarginLayoutChildren::AddLayoutWidget(Widget* pWidget, int attributes)
		{
			if (!pWidget)
			{
				return false;
			}
			if (!CheckMarginLayoutAttributes_(attributes))
			{
				return false;
			}
			widget_attributes_[pWidget] = attributes;
			return true;
		}
		bool MarginLayoutChildren::RemoveLayoutWidget(Widget* pWidget, int attributes/* = -1*/)
		{
			if (!pWidget)
			{
				return false;
			}
			if (!CheckMarginLayoutAttributes_(attributes))
			{
				return false;
			}
			auto iter = widget_attributes_.find(pWidget);
			if (widget_attributes_.end() == iter)
			{
				return false;
			}
			(*iter).second &= ~attributes;
			if (!CheckMarginLayoutAttributes_((*iter).second))
			{
				widget_attributes_.erase(iter);
			}
			return true;
		}
		bool MarginLayoutChildren::ModifyLayoutWidget(Widget* pWidget, int attributes)
		{
			if (!pWidget)
			{
				return false;
			}
			if (!CheckMarginLayoutAttributes_(attributes))
			{
				widget_attributes_.erase(pWidget);
				return true;
			}
			widget_attributes_[pWidget] = attributes;
			return true;
		}

		int MarginLayoutChildren::GetWidgetAttributes(Widget* pWidget)
		{
			auto iter = widget_attributes_.find(pWidget);
			if (widget_attributes_.end() != iter)
			{
				return (*iter).second;
			}
			return 0;
		}

		void MarginLayoutChildren::Layout(Widget* /*pWidget*/)
		{
			Widget* pWidget = GetWidget();
			if (!pWidget)
			{
				return;
			}
			RECT currentRect = {0};
			pWidget->GetAbsoluteRect(currentRect);
			if (::EqualRect(&oldRect_, &currentRect))
			{
				return;
			}
			if (!widget_attributes_.empty())
			{
				auto iter = widget_attributes_.begin();
				auto end = widget_attributes_.end();
				for (; end != iter; ++iter)
				{
					RECT rect = {0};
					(*iter).first->GetAbsoluteRect(rect);
					int attributes = (*iter).second;
					if (MARGIN_LAYOUT_KEEP_LEFT & attributes
						&& MARGIN_LAYOUT_KEEP_RIGHT & attributes)
					{
						rect.left += currentRect.left - oldRect_.left;
						rect.right += currentRect.right - oldRect_.right;
					}
					else if (MARGIN_LAYOUT_KEEP_LEFT & attributes)
					{
						::OffsetRect(&rect, currentRect.left - oldRect_.left, 0);
					}
					else if (MARGIN_LAYOUT_KEEP_RIGHT & attributes)
					{
						::OffsetRect(&rect, currentRect.right - oldRect_.right, 0);
					}
					if (MARGIN_LAYOUT_KEEP_TOP & attributes
						&& MARGIN_LAYOUT_KEEP_BOTTOM & attributes)
					{
						rect.top += currentRect.top - oldRect_.top;
						rect.bottom += currentRect.bottom - oldRect_.bottom;
					}
					else if (MARGIN_LAYOUT_KEEP_TOP & (*iter).second)
					{
						::OffsetRect(&rect, 0, currentRect.top - oldRect_.top);
					}
					else if (MARGIN_LAYOUT_KEEP_BOTTOM & (*iter).second)
					{
						::OffsetRect(&rect, 0, currentRect.bottom - oldRect_.bottom);
					}
					(*iter).first->SetAbsoluteRect(rect);
				}
			}
			::CopyRect(&oldRect_, &currentRect);
		}

		void MarginLayoutChildren::RelationshipCreated_(ObjectRelationship* pObject)
		{
			Widget* pOldWidget = GetWidget();
			MonopolisticLayoutChildren::RelationshipCreated_(pObject);
			Widget* pNewWidget = GetWidget();
			if (pNewWidget != pOldWidget
				&& pNewWidget)
			{
				pNewWidget->GetAbsoluteRect(oldRect_);
			}
		}

	} // namespace widget

} // namespace ghost