
#ifndef OBJECT_POOL_H
#define OBJECT_POOL_H

#include <list>
#include <memory>
#include <set>
#include <functional>
#include <algorithm>

namespace ghost{

	// ̰߳ȫĶ
	// ͨö̬
	// йĶ󶼱عԶգûܶdelete
	template<typename T, typename A = std::allocator<T>>
	class ObjectPool{
	public:
		typedef T Object;
		typedef A Allocator;
		typedef typename Allocator::pointer Pointer;

		enum{
			DEFAULT_BLOCK_SIZE = 32 // ĬϿСΪ32
		};

	private:
		size_t increaseBlockSize_;

		std::list<Pointer> restList_;
		std::set<Pointer> busyList_;

		std::list<Pointer> BlockList_;

	public:
		explicit ObjectPool(
			size_t initSize = DEFAULT_BLOCK_SIZE, 
			size_t increaseSize = DEFAULT_BLOCK_SIZE)
			: increaseBlockSize_(increaseSize)
		{
			if (!CheckSize_(initSize))
			{
				initSize = DEFAULT_BLOCK_SIZE;
			}
			Increase_(initSize);
		}
		~ObjectPool()
		{
			using namespace std::placeholders;

			std::for_each(
				busyList_.begin(), 
				busyList_.end(), 
				std::bind(std::mem_fn(&Allocator::destroy), &GetAllocator_(), _1));
			std::for_each(
				BlockList_.begin(), 
				BlockList_.end(), 
				std::bind(std::mem_fn(&Allocator::deallocate), &GetAllocator_(), _1, 0));
		}
		ObjectPool(ObjectPool&& rhs)
			: increaseBlockSize_(rhs.increaseBlockSize_)
			, restList_(std::move(rhs.restList_))
			, busyList_(std::move(rhs.busyList_))
			, BlockList_(std::move(rhs.BlockList_))
		{
			rhs.increaseBlockSize_ = DEFAULT_BLOCK_SIZE;
		}
		ObjectPool operator=(ObjectPool&& rhs)
		{
			ObjectPool(std::move(rhs)).swap(*this);
			return *this;
		}
		void swap(ObjectPool& rhs)
		{
			std::swap(increaseBlockSize_, rhs.increaseBlockSize_);
			restList_.swap(rhs.restList_);
			busyList_.swap(rhs.busyList_);
			BlockList_.swap(rhs.BlockList_);
		}

	private:
		ObjectPool(const ObjectPool&);
		ObjectPool& operator =(const ObjectPool&);

	public:
		// ȡ
		size_t GetIncreaseBlockSize() const{return increaseBlockSize_;}
		// 
		bool SetIncreaseBlockSize(size_t s)
		{
			if (CheckSize_(s))
			{
				increaseBlockSize_ = s;
				return true;
			}
			return false;
		}

		// ȡ
		static size_t GetBlockMaxSize(){return GetAllocator_().max_size();}

		// ֧0-10Ĺ
		Pointer Construct()
		{
			return ::new (PreConstruct_()) Object();
		}
		template<typename P1>
		Pointer Construct(const P1& p1)
		{
			return ::new (PreConstruct_()) Object(p1);
		}
		template<typename P1, typename P2>
		Pointer Construct(const P1& p1, const P2& p2)
		{
			return ::new (PreConstruct_()) Object(p1, p2);
		}
		template<typename P1, typename P2, typename P3>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3);
		}
		template<typename P1, typename P2, typename P3, typename P4>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4, p5);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4, p5, p6);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4, p5, p6, p7);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7, const P8& p8)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4, p5, p6, p7, p8);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7, const P7& p8, const P9& p9)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4, p5, p6, p7, p8, p9);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9, typename P10>
		Pointer Construct(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7, const P7& p8, const P7& p9, const P10& p10)
		{
			return ::new (PreConstruct_()) Object(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);
		}

		// عݵָ벻ɶزδٵģ򷵻һ¶ַʹòַָ͵ع
		Pointer Restructure(Pointer pPlace)
		{
			return ::new (PreRestructure_(pPlace)) Object();
		}
		template<typename P1>
		Pointer Restructure(Pointer pPlace, const P1& p1)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1);
		}
		template<typename P1, typename P2>
		Pointer Restructure(Pointer pPlace, const P1& p1, const P2& p2)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2);
		}
		template<typename P1, typename P2, typename P3>
		Pointer Restructure(Pointer pPlace, const P1& p1, const P2& p2, const P3& p3)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3);
		}
		template<typename P1, typename P2, typename P3, typename P4>
		Pointer Restructure(Pointer pPlace, const P1& p1, const P2& p2, const P3& p3, const P4& p4)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5>
		Pointer Restructure(Pointer pPlace, const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4, p5);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6>
		Pointer Restructure(Pointer pPlace, const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4, p5, p6);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7>
		Pointer Restructure(Pointer pPlace, const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4, p5, p6, p7);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8>
		Pointer Restructure(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7, const P8& p8)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4, p5, p6, p7, p8);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9>
		Pointer Restructure(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7, const P7& p8, const P9& p9)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4, p5, p6, p7, p8, p9);
		}
		template<typename P1, typename P2, typename P3, typename P4, typename P5, typename P6, typename P7, typename P8, typename P9, typename P10>
		Pointer Restructure(const P1& p1, const P2& p2, const P3& p3, const P4& p4, const P5& p5, const P6& p6, const P7& p7, const P7& p8, const P7& p9, const P10& p10)
		{
			return ::new (PreRestructure_(pPlace)) Object(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);
		}

		// ٶ
		bool Destroy(Pointer p)
		{
			auto iter = busyList_.find(p);
			if (busyList_.end() != iter)
			{
				busyList_.erase(iter);
				restList_.push_back(p);
				GetAllocator_().destroy(p);
				return true;
			}
			return false;
		}

		// ж
		void DestroyAll()
		{
			std::for_each(
				busyList_.begin(), 
				busyList_.end(), 
				std::bind(std::mem_fn(&Allocator::destroy), &allocator_, _1));
			std::copy(
				busyList_.begin(), 
				busyList_.end()
				std::back_inserter(restList_));
			busyList_.clear();
		}

		// ǷΪط
		bool CheckObject(Pointer p)
		{
			return (busyList_.end() != busyList_.find(p));
		}

	private:
		static Allocator& GetAllocator_()
		{
			static Allocator a;
			return a;
		}

		static bool CheckSize_(size_t s){return 0 < s && s <= GetBlockMaxSize();}

		void Increase_(size_t size)
		{
			Pointer p = GetAllocator_().allocate(size);
			for (size_t i = 0; i < size; ++i)
			{
				restList_.push_back(p+i);
			}
			BlockList_.push_back(p);
		}

		Pointer PreConstruct_()
		{
			if (restList_.empty())
			{
				Increase_(increaseBlockSize_);
			}
			Pointer p = restList_.front();
			restList_.pop_front();
			busyList_.insert(p);
			return p;
		}

		Pointer PreRestructure_(Pointer pPlace)
		{
			auto iter = busyList_.find(pPlace);
			if (busyList_.end() != iter)
			{
				GetAllocator_().destroy(pPlace);
				return pPlace;
			}
			return PreConstruct_();
		}

	};

} // namespace ghost

#endif // OBJECT_POOL_H