
#ifndef MULTI_OBJECT_RELATIONSHIP_H
#define MULTI_OBJECT_RELATIONSHIP_H

#include "ObjectRelationship.h"
#include <set>
#include <algorithm>
#include <functional>

namespace ghost{

	class MultiObjectRelationship : public ObjectRelationship{
	protected:
		typedef std::set<ObjectRelationship*> ReleatedObjectSet;
		ReleatedObjectSet relatedObjects_;

	public:
		MultiObjectRelationship(){}
		virtual ~MultiObjectRelationship()
		{
			DestroyAllRelationships();
		}

	public:
		void DestroyAllRelationships()
		{
			ReleatedObjectSet temp(std::move(relatedObjects_));
			std::for_each(temp.begin(), temp.end(), 
				std::bind(std::mem_fun(&ObjectRelationship::DestroyRelationship), std::placeholders::_1, this));
		}

	protected:
		virtual bool DoCreateRelationship_(ObjectRelationship* pObject)
		{
			if (!pObject)
			{
				return false;
			}
			if (!relatedObjects_.insert(pObject).second)
			{
				 return false;
			}
			pObject->CreateRelationship(this);
			return true;
		}
		virtual bool DoDestroyRelationship_(ObjectRelationship* pObject)
		{
			if (!pObject)
			{
				return false;
			}
			auto iter = relatedObjects_.find(pObject);
			if (relatedObjects_.end() == iter)
			{
				return false;
			}
			relatedObjects_.erase(iter);
			pObject->DestroyRelationship(this);
			return true;
		}

	};

} // namespace ghost

#endif // MULTI_OBJECT_RELATIONSHIP_H